// Fill out your copyright notice in the Description page of Project Settings.

#include "LoadTest.h"
#include "ImportPakFile.h"
#include "AssetStreamer.h"
#include "Singleton.h"

// Sets default values
AImportPakFile::AImportPakFile()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = false;

}

// Called when the game starts or when spawned
void AImportPakFile::BeginPlay()
{
	Super::BeginPlay();

	// Set our class up to handle clicks and touches.
	OnClicked.AddUniqueDynamic(this, &AImportPakFile::ImportPakFilePress_Mouse);
	OnBeginCursorOver.AddUniqueDynamic(this, &AImportPakFile::ImportPakFileEnter_Mouse);

	OnInputTouchBegin.AddUniqueDynamic(this, &AImportPakFile::ImportPakFilePress);
	OnInputTouchEnter.AddUniqueDynamic(this, &AImportPakFile::ImportPakFileEnter);

	Streamer = new FAssetStreamer();
	if (Streamer)
	{
		USingleton* DataInstance = Cast<USingleton>(GEngine->GameSingleton);
		Streamer->Initialize(DataInstance->GetStreamManager());
	}
}

void AImportPakFile::OnAssetStreamComplete()
{
	UE_LOG(LogClass, Log, TEXT("LoadCompleted"));
}

void AImportPakFile::OnPrepareAssetStreaming(const TArray<FStringAssetReference>& StreamedAssets)
{
	for (auto Iter(StreamedAssets.CreateConstIterator()); Iter; Iter++)
	{
		UE_LOG(LogClass, Log, TEXT("Try to load %s"), *Iter->ToString());
	}
}

// Called every frame
void AImportPakFile::Tick( float DeltaTime )
{
	Super::Tick( DeltaTime );

}

void AImportPakFile::RunLoad()
{
	Streamer->StreamPackage(FPaths::GameContentDir() + "Paks/dlc", this);
}

void AImportPakFile::ImportPakFilePress(ETouchIndex::Type FingerIndex)
{
	GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Red, FString::Printf(TEXT("Press ETouchIndex:%d"), (int32)FingerIndex));
	// We clicked or touched the tile.
	if (!UGameplayStatics::IsGamePaused(this))
	{
		RunLoad();
	}
}

void AImportPakFile::ImportPakFileEnter(ETouchIndex::Type FingerIndex)
{
	GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Red, FString::Printf(TEXT("Enter ETouchIndex:%d"), (int32)FingerIndex));
	// We have moved into the tile's space while we had a different tile selected. This is the same as pressing the tile directly.
	// Note that we need to make sure it's a different actual tile (i.e. not NULL) because deselecting a tile by touching it twice will then trigger the TileEnter event and re-select it.
	if (!UGameplayStatics::IsGamePaused(this))
	{
		ImportPakFilePress(FingerIndex);
	}
}

void AImportPakFile::ImportPakFilePress_Mouse()
{
	GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Red, FString::Printf(TEXT("Press_Mouse")));
	ImportPakFilePress(ETouchIndex::Touch1);
}

void AImportPakFile::ImportPakFileEnter_Mouse()
{
	GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Red, FString::Printf(TEXT("Enter_Mouse")));
	// This is meant to simulate finger-swiping, so ignore if the mouse isn't clicked.
	if (APlayerController* PC = UGameplayStatics::GetPlayerController(this, 0))
	{
		if (PC->IsInputKeyDown(EKeys::LeftMouseButton))
		{
			ImportPakFileEnter(ETouchIndex::Touch1);
		}
	}
}